Package: ensembldb
Type: Package
Title: Utilities to create and use Ensembl-based annotation databases
Version: 2.36.1
Authors@R: c(person(given = "Johannes", family = "Rainer",
	   email = "johannes.rainer@eurac.edu",
	   role = c("aut", "cre"),
	   comment = c(ORCID = "0000-0002-6977-7147")),
	   person(given = "Tim", family = "Triche",
	   email = "tim.triche@usc.edu",
	   role = "ctb"),
	   person(given = "Christian", family = "Weichenberger",
	   email = "christian.weichenberger@eurac.edu",
	   role = "ctb",
	   comment = c(ORCID = "0000-0002-2176-0274")),
	   person(given = "Sebastian", family = "Gibb",
	   email = "mail@sebastiangibb.de",
	   role = "ctb",
	   comment = c(ORCID = "0000-0001-7406-4443")),
	   person(given = "Laurent", family = "Gatto",
	   email = "lg390@cam.ac.uk",
	   role = "ctb",
	   comment = c(ORCID = "0000-0002-1520-2268")),
       person(given = "Boyu", family = "Yu",
	   email = "boyu.yu.tim@gmail.com",
	   role = "ctb"))
Author: Johannes Rainer <johannes.rainer@eurac.edu> with contributions
	from Tim Triche, Sebastian Gibb, Laurent Gatto
	Christian Weichenberger and Boyu Yu.
Maintainer: Johannes Rainer <johannes.rainer@eurac.edu>
URL: https://github.com/jorainer/ensembldb
BugReports: https://github.com/jorainer/ensembldb/issues
Imports: methods, RSQLite (>= 1.1), DBI, Biobase, Seqinfo, GenomeInfoDb
        (>= 1.45.5), AnnotationDbi (>= 1.31.19), rtracklayer (>=
        1.69.1), S4Vectors (>= 0.23.10), Rsamtools, IRanges (>=
        2.13.24), ProtGenerics, Biostrings (>= 2.77.2), curl
Depends: R (>= 3.5.0), BiocGenerics (>= 0.15.10), GenomicRanges (>=
        1.61.1), GenomicFeatures (>= 1.61.4), AnnotationFilter (>=
        1.5.2)
Suggests: BiocStyle, knitr, EnsDb.Hsapiens.v86 (>= 0.99.8), testthat,
        BSgenome.Hsapiens.NCBI.GRCh38, ggbio (>= 1.24.0), Gviz (>=
        1.20.0), rmarkdown, AnnotationHub
Enhances: RMariaDB, shiny
VignetteBuilder: knitr
Description: The package provides functions to create and use
    transcript centric annotation databases/packages. The
    annotation for the databases are directly fetched from Ensembl
    using their Perl API. The functionality and data is similar to
    that of the TxDb packages from the GenomicFeatures package,
    but, in addition to retrieve all gene/transcript models and
    annotations from the database, ensembldb provides a filter
    framework allowing to retrieve annotations for specific entries
    like genes encoded on a chromosome region or transcript models
    of lincRNA genes. EnsDb databases built with ensembldb contain
    also protein annotations and mappings between proteins and
    their encoding transcripts. Finally, ensembldb provides
    functions to map between genomic, transcript and protein
    coordinates.
Collate: 'Classes.R' 'Deprecated.R' 'Generics.R' 'Methods-Filter.R'
        'Methods.R' 'dbhelpers.R' 'functions-EnsDb.R'
        'functions-Filter.R' 'functions-create-EnsDb.R'
        'functions-utils.R' 'proteinToX.R' 'transcriptToX.R'
        'genomeToX.R' 'select-methods.R' 'seqname-utils.R' 'zzz.R'
biocViews: Genetics, AnnotationData, Sequencing, Coverage
License: LGPL
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/ensembldb
git_branch: RELEASE_3_23
git_last_commit: 5c98c2b
git_last_commit_date: 2026-05-25
Repository: Bioconductor 3.23
Date/Publication: 2026-05-25
NeedsCompilation: no
Packaged: 2026-05-25 21:39:27 UTC; biocbuild
